'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _extends = _interopDefault(require('@babel/runtime/helpers/extends'));
var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));
var reactSpring = require('react-spring');
var reactUseGesture = require('react-use-gesture');
var useResizeObserver = _interopDefault(require('use-resize-observer'));
require('intersection-observer');

var slidesWrapperStyle = function slidesWrapperStyle(vertical) {
  return {
    display: 'flex',
    height: '100%',
    width: '100%',
    flexWrap: 'nowrap',
    alignItems: 'stretch',
    position: 'relative',
    WebkitUserSelect: 'none',
    userSelect: 'none',
    WebkitTouchCallout: 'none',
    flexDirection: vertical ? 'column' : 'row',
    touchAction: vertical ? 'pan-x' : 'pan-y'
  };
};

var clamp = function clamp(num, _clamp, higher) {
  return Math.min(Math.max(num, _clamp), higher);
};

function Slider(_ref) {
  var children = _ref.children,
      index = _ref.index,
      onIndexChange = _ref.onIndexChange,
      className = _ref.className,
      style = _ref.style,
      slideStyle = _ref.slideStyle,
      enabled = _ref.enabled,
      trail = _ref.trail,
      vertical = _ref.vertical,
      draggedScale = _ref.draggedScale,
      draggedSpring = _ref.draggedSpring,
      trailingSpring = _ref.trailingSpring,
      trailingDelay = _ref.trailingDelay,
      onDragStart = _ref.onDragStart,
      onDragEnd = _ref.onDragEnd;
  var slideStyleFunc = typeof slideStyle === 'function' ? slideStyle : function () {
    return slideStyle;
  };
  var root = React.useRef();

  var _useResizeObserver = useResizeObserver({
    ref: root
  }),
      width = _useResizeObserver.width,
      height = _useResizeObserver.height;

  var axis = vertical ? 'y' : 'x';
  var size = vertical ? height : width;
  var indexRef = React.useRef(index);
  var restPos = React.useRef(0);
  var velocity = React.useRef(0);
  var visibleIndexes = React.useRef(new Set());
  var firstVisibleIndex = React.useRef(0);
  var lastVisibleIndex = React.useRef(0);

  var cb = function cb(slides) {
    slides.forEach(function (_ref2) {
      var isIntersecting = _ref2.isIntersecting,
          target = _ref2.target;
      return visibleIndexes.current[isIntersecting ? 'add' : 'delete'](~~target.attributes['data-index'].value);
    });
    var visibles = Array.from(visibleIndexes.current).sort();
    firstVisibleIndex.current = visibles[0];
    lastVisibleIndex.current = visibles[visibles.length - 1];
  };

  var observer = React.useRef(null);
  if (!observer.current) observer.current = new IntersectionObserver(cb);
  React.useEffect(function () {
    Array.from(root.current.children).forEach(function (t) {
      return observer.current.observe(t);
    });
  }, [children.length, root]);
  React.useEffect(function () {
    return function () {
      return observer.current.disconnect();
    };
  }, []);

  var _useSprings = reactSpring.useSprings(children.length, function (i) {
    return {
      x: vertical ? 0 : restPos.current,
      y: vertical ? restPos.current : 0,
      s: 1,
      zIndex: 0,
      immediate: function immediate(key) {
        return key === 'zIndex';
      }
    };
  }, [children.length]),
      springs = _useSprings[0],
      set = _useSprings[1];

  React.useEffect(function () {
    if (!width || !height) return;

    if (vertical) {
      var _root$current$childre = root.current.children[index],
          offsetTop = _root$current$childre.offsetTop,
          offsetHeight = _root$current$childre.offsetHeight;
      restPos.current = Math.round(-offsetTop + (height - offsetHeight) / 2);
    } else {
      var _root$current$childre2 = root.current.children[index],
          offsetLeft = _root$current$childre2.offsetLeft,
          offsetWidth = _root$current$childre2.offsetWidth;
      restPos.current = Math.round(-offsetLeft + (width - offsetWidth) / 2);
    }

    if (indexRef.current === index) {
      set(function () {
        var _ref3;

        return _ref3 = {}, _ref3[axis] = restPos.current, _ref3.s = 1, _ref3.config = function config(key) {
          return key === axis && _extends({}, trailingSpring, {
            velocity: velocity.current
          });
        }, _ref3;
      });
    } else {
      var dir = index < indexRef.current ? -1 : 1;
      var firstToMove = dir > 0 ? firstVisibleIndex.current : lastVisibleIndex.current;
      set(function (i) {
        var _ref4;

        return _ref4 = {}, _ref4[axis] = restPos.current, _ref4.s = 1, _ref4.config = function config(key) {
          return key === axis && trailingSpring;
        }, _ref4.delay = i * dir < firstToMove * dir ? 0 : Math.abs(firstToMove - i) * trailingDelay, _ref4;
      });
    }

    indexRef.current = index;
  }, [index, set, root, vertical, axis, height, width, trail, trailingSpring, draggedSpring, trailingDelay]);
  var bind = reactUseGesture.useDrag(function (_ref5) {
    var first = _ref5.first,
        last = _ref5.last,
        _ref5$vxvy = _ref5.vxvy,
        vx = _ref5$vxvy[0],
        vy = _ref5$vxvy[1],
        _ref5$delta = _ref5.delta,
        dx = _ref5$delta[0],
        dy = _ref5$delta[1],
        _ref5$movement = _ref5.movement,
        movX = _ref5$movement[0],
        movY = _ref5$movement[1],
        _ref5$args = _ref5.args,
        pressedIndex = _ref5$args[0],
        _ref5$memo = _ref5.memo,
        memo = _ref5$memo === void 0 ? springs[pressedIndex][axis].get() : _ref5$memo;
    var v = vertical ? vy : vx;
    var dir = -Math.sign(vertical ? dy : dx);
    var mov = vertical ? movY : movX;

    if (first) {
      set(function (i) {
        var _ref6;

        return _ref6 = {}, _ref6[axis] = memo + mov, _ref6.s = draggedScale, _ref6.config = function config(key) {
          return key === axis && i === pressedIndex ? draggedSpring : trailingSpring;
        }, _ref6.zIndex = i === pressedIndex ? 10 : 0, _ref6;
      });
      onDragStart && onDragStart(pressedIndex);
    } else if (last) {
      if (Math.abs(mov) > size / 2 || Math.abs(v) > 0.3) {
        indexRef.current = clamp(indexRef.current + (mov > 0 ? -1 : 1), 0, children.length - 1);
      }

      if (index !== indexRef.current) {
        velocity.current = v;
        requestAnimationFrame(function () {
          return onIndexChange(indexRef.current);
        });
      } else set(function () {
          var _ref7;

          return _ref7 = {}, _ref7[axis] = restPos.current, _ref7.s = 1, _ref7.config = function config(key) {
            return key === axis && trailingSpring;
          }, _ref7;
        });

      onDragEnd && onDragEnd(pressedIndex);
    } else {
        var firstToMove = dir > 0 ? firstVisibleIndex.current : lastVisibleIndex.current;
        set(function (i) {
          var _ref8;

          return _ref8 = {}, _ref8[axis] = mov + memo, _ref8.delay = i * dir < firstToMove * dir || i === pressedIndex ? 0 : Math.abs(firstToMove - i) * trailingDelay, _ref8.config = function config(key) {
            return key === axis && i === pressedIndex ? draggedSpring : trailingSpring;
          }, _ref8;
        });
      }

    return memo;
  }, {
    enabled: enabled,
    axis: axis
  });
  return React__default.createElement("div", {
    className: className,
    style: style
  }, React__default.createElement("div", {
    ref: root,
    style: slidesWrapperStyle(vertical)
  }, springs.map(function (_ref9, i) {
    var _extends2;

    var pos = _ref9[axis],
        s = _ref9.s,
        zIndex = _ref9.zIndex;
    return React__default.createElement(reactSpring.animated.div, _extends({}, bind(i), {
      key: i,
      "data-index": i,
      style: _extends({}, slideStyleFunc(i), (_extends2 = {
        zIndex: zIndex
      }, _extends2[axis] = pos, _extends2.scale = s, _extends2.display = 'flex', _extends2.alignItems = 'center', _extends2.willChange = 'transform', _extends2))
    }), children[i]);
  })));
}
Slider.defaultProps = {
  className: undefined,
  style: undefined,
  enabled: true,
  trail: true,
  vertical: false,
  draggedScale: 1,
  draggedSpring: {
    tension: 1200,
    friction: 40
  },
  trailingSpring: {
    mass: 1,
    tension: 280,
    friction: 60
  },
  trailingDelay: 50,
  onDragStart: undefined,
  onDragEnd: undefined
};
Slider.propTypes = {
  children: PropTypes.node.isRequired,
  index: PropTypes.number.isRequired,
  onIndexChange: PropTypes.func.isRequired,
  className: PropTypes.string,
  style: PropTypes.object,
  slideStyle: PropTypes.object,
  enabled: PropTypes.bool,
  vertical: PropTypes.bool,
  trail: PropTypes.bool,
  draggedScale: PropTypes.number,
  draggedSpring: PropTypes.object,
  trailingSpring: PropTypes.object,
  onDragStart: PropTypes.func,
  onDragEnd: PropTypes.func
};

exports.Slider = Slider;
